/* 
   This file is part of KDE project.
   Copyright (C) 2004 Dawit Alemayehu <adawit@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation, version 2.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include <qdir.h>
#include <qtimer.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qgroupbox.h>
#include <qpushbutton.h>

#include <kdebug.h>
#include <klocale.h>
#include <kprogress.h>
#include <klineedit.h>
#include <klibloader.h>
#include <kfiledialog.h>
#include <kmessagebox.h>
#include <kstandarddirs.h>
#include <ksqueezedtextlabel.h>

#include "kbrokenlibdetector.hpp"
#include "kbrokenlibdetector_ui.h"


KBrokenLibDetector::KBrokenLibDetector()
                   :KUniqueApplication(), _run(false), _widget(0)
{
}
  
int KBrokenLibDetector::newInstance()
{
  if (!_widget)
  {
    _widget = new KBrokenLibDetectorUI;
    
    connect (this, SIGNAL(lastWindowClosed()), SLOT(slotQuit()));
    connect (_widget->pbStart, SIGNAL(clicked()), SLOT(slotStart()));
    connect (_widget->pbStop, SIGNAL(clicked()), SLOT(slotStop()));
    connect (_widget->pbDetails, SIGNAL(clicked()), SLOT(slotDetails()));
    connect (_widget->lsBrokenLibs, SIGNAL(doubleClicked(QListBoxItem*)), SLOT(slotDetails()));
    connect (_widget->lsBrokenLibs, SIGNAL(highlighted(int)), SLOT(slotHighlighted(int)));
    
    KStandardDirs dirs;
    _libPath = dirs.findResourceDir("lib", "libkdecore.la");
    _widget->leKDEPath->setText(_libPath);
  }
      
  _widget->show();
  
  return 0;
}

KBrokenLibDetector::~KBrokenLibDetector()
{
}

void KBrokenLibDetector::slotStart()
{
  _run = true;
  reset();
  updateButtons();
  QTimer::singleShot(0, this, SLOT(startChecking()));
}

void KBrokenLibDetector::slotStop()
{
  _run = false;
  reset();
  updateButtons();
}

void KBrokenLibDetector::slotQuit()
{
  slotStop();
  quit();
}

void KBrokenLibDetector::slotDetails()
{
  QString msg;
  QString lib = _widget->lsBrokenLibs->currentText();
   
  if (!lib.isEmpty())
    msg = i18n("<qt><b>%1</b><br/>%2</qt>").arg(lib).arg(_errorMap[lib]);
  else
    msg = i18n("<qt>Not available</qt>");
  
  KMessageBox::information(_widget, msg, i18n("Error Details"));
}

void KBrokenLibDetector::findLibs( const QString& libDir, InfoList* libList, bool recurse )
{
  QDir dir( libDir );
  
  const QFileInfoList* libs = dir.entryInfoList(QString::fromAscii("*.la"), 
                                                QDir::Files | QDir::NoSymLinks);
  
  if (!libs)
    return;
    
  QFileInfoListIterator it ( *libs );
  
  for( ; it != 0 && _run; ++it)
  {
    LibInfo info;
    
    info.baseName = it.current()->baseName( true );
    info.fullPath = it.current()->absFilePath();
    
    libList->append( info );
  }
    
  if (recurse)
  {
    QStringList subDirs = dir.entryList(QString::fromAscii("*"), QDir::Dirs);
    
    for ( QStringList::Iterator it = subDirs.begin(); it != subDirs.end() && _run; ++it )
    {
      if (*it == "." || *it == "..")
        continue;
        
      dir.cd( *it );
      
      const QFileInfoList* libs = dir.entryInfoList(QString::fromAscii("*.la"), 
                                                    QDir::Files | QDir::NoSymLinks);
      
      if (!libs)
        continue;
        
      QFileInfoListIterator it2 ( *libs );
      
      for( ; it2 != 0 && _run; ++it2 )
      {
        LibInfo info;
        
        info.baseName = it2.current()->baseName( true );
        info.fullPath = it2.current()->absFilePath();
        
        libList->append( info );
      }
      
      dir.cdUp();
      processEvents();
    }
  }
}

void KBrokenLibDetector::startChecking()
{
  InfoList libs;  
  findLibs( _libPath, &libs,  _widget->cbRecurse->isChecked());
  
  int count = 1;
  _widget->piCompleted->setTotalSteps(libs.count());
    
  for ( InfoList::Iterator it = libs.begin(); it != libs.end() && _run; ++it )
  {
    _widget->leTesting->setText( (*it).fullPath );
    _widget->lbCompleted->setText( QString("Status: %1 of %2 completed").arg(count).arg(libs.count()) );
    _widget->piCompleted->setProgress( count++ );
    
    kdDebug() << "BaseName: " << (*it).baseName << endl;
    if (!KLibLoader::self()->library( QFile::encodeName((*it).baseName)) )
    {
      addBrokenLib( (*it).fullPath );
      continue;
    }        
    
    // Unload the library once we are done with it...
    KLibLoader::self()->unloadLibrary( QFile::encodeName((*it).baseName) );
    processEvents();    
  }
  
  // If we completed sucessfully
  QString status;
  if (_run)
    status = i18n("Status: Done...");
  else
    status = i18n("Status: Cancelled...");

  _widget->lbCompleted->setText( status );
    
  _run = false;
  updateButtons();  
}

void KBrokenLibDetector::updateButtons()
{
  _widget->pbStart->setEnabled(!_run);
  _widget->cbRecurse->setEnabled(!_run);
  _widget->pbStop->setEnabled(_run);
}

void KBrokenLibDetector::reset()
{
  _widget->piCompleted->reset();
  _widget->leTesting->clear();
  _widget->lsBrokenLibs->clear();
  _widget->lsBrokenLibs->setEnabled( false );
  _widget->pbDetails->setEnabled ( false );
  _errorMap.clear();  
}

void KBrokenLibDetector::addBrokenLib( const QString& lib)
{
  if (!_widget->lsBrokenLibs->isEnabled())
    _widget->lsBrokenLibs->setEnabled( true );
    
  _widget->lsBrokenLibs->insertItem(lib);
  
  // Store the details of the error message...
  _errorMap[lib] = KLibLoader::self()->lastErrorMessage();
}

void KBrokenLibDetector::slotHighlighted(int)
{
  _widget->pbDetails->setEnabled(_widget->lsBrokenLibs->count() > 0);
}
